Lua_PackBatchWindow = class("Lua_PackBatchWindow", Lua_Window);
function Lua_PackBatchWindow:ctor()
    self:superCall("ctor");

    self.windowName = "Lua_PackBatchWindow";
    self.skinName = "PackBatchWindow_skin";
    self.showBlackMask = true;
    self.isClickClose = true;
end

function Lua_PackBatchWindow:buildShowItem()
    self:superCall("buildShowItem");
    self.selectedDic = {};
    self.selectedArr = Array();

    self:addLuaEvent("PACK_BATCH_UPDATE_SELECT", self, self.updateBatchSelectInfo);--更新批量选择列表
    self:addLuaEvent("PACK_FILTER_LIST_CLOSE", self, self.onDownMenuClick)
    self:addLuaEvent("PACK_FILTER", self, self.updateFilterDic);
    self:addLuaEvent("PACK_BATCH_PLUS", self, self.onItemPlus);
    self:addLuaEvent("PACK_BATCH_LESS", self, self.onItemLess);
    
    self.itemList = Lua_DragList.new(self.content, 620, 420, Lua_AxisDefine.Vertical);
    self.itemList:setLocation(1, 94, 0);
    self.itemList.canLongPress = true;
    self.itemList.customParam = self.selectedDic;
    self.itemList:setListCell(Lua_PackBatchItem, 112, 140, 5);
    self.itemList:setCellSpacing(15, 10);

    self.getList = Lua_DragList.new(self.content, 620, 112, Lua_AxisDefine.Horizontal);
    self.getList:setLocation(0, -315, 0);
    self.getList:setListCell(Lua_PackBatchGetItem, 112, 112, 1);
    self.getList:setCellSpacing(15, 0);

    self.bDropDownMenu = false
    if self.bDropDownMenu then
        self.menuArrow:setAltas("common_altas_1", "xui_common_button_018")
    else
        self.menuArrow:setAltas("common_altas_1", "xui_common_button_017")
    end
    self.menuBg:addClickFuntion(self, self.onDownMenuClick)

    self.btnFilter.txt:setLocation(0, 0, 0);
    self.btnFilter:addClickFuntion(self, self.onBtnFilterClick);
    self.checkHelp:addClickFuntion(self, self.onCheckHelp);
    self.btnOneKey:addClickFuntion(self, self.onBtnOneKeyClick);
    self.btnFunc:addClickFuntion(self, self.onBtnFuncClick);
end

function Lua_PackBatchWindow:dispose()
    if self.itemList ~= nil then
        self.itemList:dispose();
        self.itemList = nil;
    end
    if self.getList ~= nil then
        self.getList:dispose();
        self.getList = nil;
    end
    if self.itemArr ~= nil then
        self.itemArr:clear();
        self.itemArr = nil;
    end
    if self.tvoArr ~= nil then
        self.tvoArr:clear();
        self.tvoArr = nil;
    end
    if self.selectedArr ~= nil then
        self.selectedArr:clear();
        self.selectedArr = nil;
    end
    self.selectedDic = nil;
    self.filterDic = nil;
    self:superCall("dispose");
end

function Lua_PackBatchWindow:showWin(param)
    self:superCall("showWin", param);

    if param.batchType == 1 then -- 分解
        if param.packType == Lua_PackDefine.TYPE_CHIP then
            self.titleName:setLangValue(309071);
        else
            self.titleName:setLangValue(309069);
        end
        self.btnFunc:setLangValue(309013);
    elseif param.batchType == 2 then -- 出售
        self.titleName:setLangValue(309070);
        self.btnFunc:setLangValue(309014);
    end

    if param.packType == Lua_PackDefine.TYPE_EQUIP then
        self.selectedText.pivotValue = 4
        self.selectedText.alignmentValue = 4
        self.selectedText:setLocation(181, -379, 0)
        self.btnFilter:setEnable(false)
        self.line_1:setLocation(0, -174, 0)
        self.text_1:setLocation(0, -174, 0)
        self.line_2:setLocation(0, -343.5, 0)
        self.menuObject:SetActive(true)
        self.getList:setLocation(0, -268.5, 0)

        self.filterDic = {}
        self.filterDic["colorStar"] = {0, 0}
        self:updateFilterDic(self.filterDic)
    else
        self.selectedText.pivotValue = 3
        self.selectedText.alignmentValue = 3
        self.selectedText:setLocation(-308, -169, 0)
        self.btnFilter:setEnable(true)
        self.line_1:setLocation(0, -221.5, 0)
        self.text_1:setLocation(0, -221.5, 0)
        self.line_2:setLocation(0, -390, 0)
        self.menuObject:SetActive(false)
        self.getList:setLocation(0, -315, 0)

        self:updateFilterDic()
    end
end

function Lua_PackBatchWindow:updateFilterDic(filterDic)
    if self.showData.packType == Lua_PackDefine.TYPE_EQUIP then
        self.menuName.text = DataCenter.packData:getFilterName(filterDic["colorStar"])
    end
    self.filterDic = filterDic;
    self:updateListInfo(filterDic);
end

function Lua_PackBatchWindow:updateListInfo(filterDic)
    if self.itemArr then
        self.itemArr:clear()
        self.itemArr = nil;
    end
    local packType = self.showData.packType;
    local batchType = self.showData.batchType;
    if packType == Lua_PackDefine.TYPE_RUNSE then
        self.itemArr = DataCenter.runesData:getBatchArrByFilter(batchType, filterDic);
    else
        self.itemArr = DataCenter.packData:getBatchArrByFilter(packType, batchType, filterDic);
    end

    self:updateTotalCount();
    self.itemList:setArrayData(self.itemArr);
    self:showNone(self.itemArr:getSize() <= 0);
    self.selectedCount = 0;
    self:updateFilterList();
end

function Lua_PackBatchWindow:showNone(bShow)
    if bShow then
        if self.noneIcon == nil then
            self.noneIcon = UICreate.createUIAtlasImage(self.content, "common_altas_3", "xui_common_widget_012", 0, 120, 0, 0);
            self.noneText = UICreate.createLabel(self.content, nil, 0, 88.5, 100, 40, 40);
            self.noneText.pivotValue = 4;
            self.noneText.fontName = Lua_MyUISettings.huakangFontName;
            self.noneText.alignmentValue = 4;
            self.noneText.horizontalOverflowValue = 1;
            self.noneText.colorStr = Lua_ColorDefine.transColorStr("938680");
            self.noneText:setLangValue(309000);
        end
        self.noneIcon:setEnable(true);
        self.noneText:setEnable(true);
    else
        if self.noneIcon then
            self.noneIcon:setEnable(false);
            self.noneText:setEnable(false);
        end
    end
end

function Lua_PackBatchWindow:updateTotalCount()
    self.totalCount = 0;
    if self.itemArr then
        local tvo;
        for i = 1, self.itemArr:getSize() do
            tvo = DataCenter.packData:getItemVoByUnId(self.itemArr[i])
            if tvo == nil then
                tvo = DataCenter.runesData:getRunesVoByUid(self.itemArr[i])
            end
            if tvo.lockState ~= true then
                self.totalCount = self.totalCount + tvo.count;
            end
        end
    end
end

function Lua_PackBatchWindow:onItemPlus(data)
    if self.selectedDic[data.unique_id] == nil then
        self.selectedDic[data.unique_id] = 1;
        self.selectedArr:add(data.unique_id);
    else
        local count = self.selectedDic[data.unique_id] + 1;
        if count >= data.count then
            count = data.count;
        end
        self.selectedDic[data.unique_id] = count;
    end

    self:updateShowList();
    self:updateGetList();
end

--装备上锁状态改变更新列表
function Lua_PackBatchWindow:updateBatchSelectInfo(unique_id)
    local data = DataCenter.packData:getItemVoByUnId(unique_id);
    if data.main_type == Lua_PackDefine.EQUIP then --装备
        self:updateTotalCount()
        self.itemList:updateShowList()
        if data.lockState then
            self:onItemLess(data)
        end
    end
end

function Lua_PackBatchWindow:onItemLess(data)
    if self.selectedDic[data.unique_id] == nil then
        return;
    end

    local count = self.selectedDic[data.unique_id] - 1;
    self.selectedDic[data.unique_id] = count;
    if count <= 0 then
        self.selectedDic[data.unique_id] = nil;
        self.selectedArr:removeItem(data.unique_id);
    end

    self:updateShowList();
    self:updateGetList();
end

function Lua_PackBatchWindow:updateFilterList()
    if self.filterDic == nil then
        return;
    end

    local tvo;
    if self.showData.packType == Lua_PackDefine.TYPE_RUNSE then
        for k,v in pairs(self.selectedDic) do
            tvo = DataCenter.runesData:getRunesVoByUid(k);
            if not DataCenter.packData:checkVoIsMeetFilter(self.filterDic, tvo) then
                self.selectedDic[k] = nil;
                self.selectedArr:removeItem(k);
            end
        end
    else
        for k,v in pairs(self.selectedDic) do
            tvo = DataCenter.packData:getItemVoByUnId(k);
            if not DataCenter.packData:checkVoIsMeetFilter(self.filterDic, tvo) then
                self.selectedDic[k] = nil;
                self.selectedArr:removeItem(k);
            end
        end
    end

    self:updateShowList();
    self:updateGetList();
end

function Lua_PackBatchWindow:updateShowList()
    local showList = self.itemList.showList;
    for i = 1, showList:getSize() do
        showList[i]:updateSelected();
    end

    self.selectedCount = self:getSelectedCount();
    self.selectedText.text = LanUtil.formateStr(309040, tostring(self.selectedCount));
    if self.selectedCount > 0 and self.selectedCount >= self.totalCount then
        self.btnOneKey:setLangValue(309064);
    else
        self.btnOneKey:setLangValue(309063);
    end
end

function Lua_PackBatchWindow:getSelectedCount()
    local count = 0;
    local uniqueId;
    for i = 1, self.selectedArr:getSize() do
        uniqueId = self.selectedArr[i];
        count = count + self.selectedDic[uniqueId];
    end
    return count;
end

function Lua_PackBatchWindow:updateGetList()
    local dic = {};
    if self.tvoArr == nil then
        self.tvoArr = Array();
    end
    local len = self.selectedArr:getSize();
    local uniqueId;
    local count;
    local tvo;
    local getVo;
    for i = 1, len do
        uniqueId = self.selectedArr[i];
        count = self.selectedDic[uniqueId] == nil and 0 or self.selectedDic[uniqueId];
        if self.showData.packType == Lua_PackDefine.TYPE_RUNSE then
            tvo = DataCenter.runesData:getRunesVoByUid(uniqueId);
        else
            tvo = DataCenter.packData:getItemVoByUnId(uniqueId);
        end
        if self.showData.batchType == 1 then -- 分解
            if tvo ~= nil and tvo.decompose ~= nil then
                if tvo.main_type == Lua_PackDefine.EQUIP then --装备
                    local id;
                    local num;
                    local arr = tvo.decompose;
                    for i=1, arr:getSize() do
                        id = arr[i].id;
                        num = arr[i].count;
                        if dic[id] == nil then
                            getVo = self.tvoArr[j];
                            if getVo == nil then
                                getVo = Lua_ToolVo.createToolVo(id);
                                self.tvoArr:add(getVo);
                            end
                            getVo:setData(id, num);
                            dic[id] = getVo;
                        else
                            dic[id].count = dic[id].count + num;
                        end
                    end
                else
                    for j = 1, #tvo.decompose do
                        local id = tvo.decompose[j][1];
                        local num = tvo.decompose[j][2];
                        if dic[id] == nil then
                            getVo = self.tvoArr[j];
                            if getVo == nil then
                                getVo = Lua_ToolVo.createToolVo(id);
                                self.tvoArr:add(getVo);
                            end
                            getVo:setData(id, count * num);
                            dic[id] = getVo;
                        else
                            dic[id].count = dic[id].count + count * num;
                        end
                    end
                end
            end
        elseif self.showData.batchType == 2 then -- 出售
            if tvo ~= nil and tvo.money_id > 0 then
                if dic[tvo.money_id] == nil then
                    getVo = self.tvoArr[1];
                    if getVo == nil then
                        getVo = Lua_ToolVo.createToolVo(tvo.money_id);
                        self.tvoArr:add(getVo);
                    end
                    getVo:setData(tvo.money_id, count * tvo.sys_price);
                    dic[tvo.money_id] = getVo;
                else
                    dic[tvo.money_id].count = dic[tvo.money_id].count + count * tvo.sys_price;
                end
            end
        end
    end

    local arr = Array();
    for k,v in pairs(dic) do
        arr:add(v)
    end
    len = arr:getSize();
    if self.getLen ~= len then -- 长度不同才修改显示区域大小
        self.getLen = len;
        if len > 5 then
            self.getList:setContentRect(620, 112);
        else
            self.getList:setContentRect(len * 112 + (len - 1) * 15, 112);
        end
    end
    self.getList:setArrayData(arr);
    self.getList.canDrag = not self.getList.isTheEnd;

    arr:clear();
    arr = nil;
    dic = nil;
end

function Lua_PackBatchWindow:onDownMenuClick(go)
    self.bDropDownMenu = not self.bDropDownMenu
    self:updateDownMenuState()
end

function Lua_PackBatchWindow:updateDownMenuState()
    if self.bDropDownMenu then
        self.menuArrow:setAltas("common_altas_1", "xui_common_button_018")
    else
        self.menuArrow:setAltas("common_altas_1", "xui_common_button_017")
    end
    if self.bDropDownMenu then
        Lua_WindowUtil.openWin("Lua_PackFilterListWindow", Lua_WinParam.ACTION_OPEN, { packType = self.showData.packType, filterDic = self.filterDic})
    else
        Lua_WindowUtil.openWin("Lua_PackFilterListWindow", Lua_WinParam.ACTION_CLOSE)
    end
end

function Lua_PackBatchWindow:onBtnFilterClick(go)
    if self.showData == nil or (self.clickTime ~= nil and getTime() - self.clickTime < 0.3) then
        return;
    end
    self.clickTime = getTime();

    Lua_WindowUtil.openWin("Lua_PackFilterWindow", Lua_WinParam.ACTION_OPEN, { packType = self.showData.packType, filterDic = self.filterDic});
end

function Lua_PackBatchWindow:onCheckHelp(go)
    local tipsData;
    if self.showData.batchType == 2 then
        tipsData = { des = LanUtil.getValue(309061), posX = -40, posY = 225, needAdapt = false };
    else
        tipsData = { des = LanUtil.getValue(309060), posX = -40, posY = 225, needAdapt = false };
    end
    Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_PackBatchWindow:onBtnOneKeyClick(go)
    if self.selectedCount >= self.totalCount then
        -- self.selectedDic = nil; self.selectedDic = {}; 都不能修改item中customParam的引用
        local len = self.selectedArr:getSize();
        local uniqueId;
        while len > 0 do
            uniqueId = self.selectedArr:remove();
            self.selectedDic[uniqueId] = nil;
            len = len - 1;
        end
    else
        local tvo;
        for i = 1, self.itemArr:getSize() do
            tvo = DataCenter.packData:getItemVoByUnId(self.itemArr[i])
            if tvo == nil then
                tvo = DataCenter.runesData:getRunesVoByUid(self.itemArr[i])
            end
            local bool = true;
            if tvo.main_type == Lua_PackDefine.EQUIP and tvo.lockState then
                bool = false;
            end
            if bool then
                if self.selectedDic[tvo.unique_id] == nil then
                    self.selectedArr:add(tvo.unique_id);
                end
                self.selectedDic[tvo.unique_id] = tvo.count;
            end
        end
    end

    self:updateShowList();
    self:updateGetList();
end

function Lua_PackBatchWindow:onBtnFuncClick(go)
    if self.selectedArr == nil or self.selectedArr:getSize() <= 0 then
        OperateAlert.showLangValue(309062);
        return;
    end

    if self.showData.batchType == 1 then -- 分解
        if self.showData.packType == Lua_PackDefine.TYPE_RUNSE then
            Lua_ProxyManager.getProxy("Lua_RunesProxy"):sDecomposeRunes(self.selectedDic,self.showData.packType);
        else
            Lua_ProxyManager.getProxy("Lua_PackProxy"):sOneKeyDecompose(self.selectedDic,self.showData.packType);
        end
    elseif self.showData.batchType == 2 then -- 出售
        Lua_ProxyManager.getProxy("Lua_PackProxy"):sOneKeySell(self.showData.packType, self.selectedDic);
    end
end




















Lua_PackBatchItem = class("Lua_PackBatchItem", Lua_ListCell)
function Lua_PackBatchItem:ctor(p)
    self:superCall("ctor", p);
end

function Lua_PackBatchItem:buildShowItem()
    self:superCall("buildShowItem");

    self.itemIcon = Lua_ItemIcon.new(self.content);
    self.itemIcon:setLocation(0, 14, 0);
    self.itemIcon:setScale(0.9, 0.9, 0.9);

    self.countText = UICreate.createLabel(self.content, nil, 0, -59.5, 100, 31, 22);
    self.countText.pivot = UIUtils.Pivot.Center;
    self.countText.alignmentValue = 4;
    self.countText.horizontalOverflowValue = 1;
    self.countText.colorStr = Lua_ColorDefine.transColor("7A6A5E");
end

function Lua_PackBatchItem:dispose()
    self:removeLessTimer();
    if self.itemIcon ~= nil then
        self.itemIcon:dispose();
        self.itemIcon = nil;
    end
    self.customParam = nil;
    self.data = nil;
    self:superCall("dispose");
end

function Lua_PackBatchItem:setCustomParam(customParam)
    self.customParam = customParam;
end

function Lua_PackBatchItem:setData(data)
    self.uniqueId = data
    self:updateData()
end

function Lua_PackBatchItem:updateData()
    self.data = DataCenter.packData:getItemVoByUnId(self.uniqueId)
    if self.data == nil then
        self.data = DataCenter.runesData:getRunesVoByUid(self.uniqueId)
    end
    self.itemIcon:setData(self.data);
    self.itemIcon:showNum(0);
    self:updateSelected();
end

function Lua_PackBatchItem:updateSelected()
    local count = 0;
    if self.customParam ~= nil and self.data ~= nil then
        count = self.customParam[self.data.unique_id];
    end
    self.selectedCount = count == nil and 0 or count;
    self.countText.text = LanUtil.formateStr(10044, tostring(self.selectedCount), tostring(self.data.count));
    if self.selectedCount > 0 then
        self.itemIcon:setMask(true);
        if self.btnLess == nil then
            self.btnLess = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_button_019", 28, 50, 0, 0);
            self.btnLess:setScale(0.9, 0.9, 0.9);
            self.btnLess:addMousDownFuntion(self, self.onBtnLessClick);
            self.btnLess:addMouseUpFuntion(self, self.onBtnLessUpClick);
        end
        self.btnLess:setEnable(true);
    else
        self.itemIcon:setMask(false);
        if self.btnLess ~= nil then
            self.btnLess:setEnable(false);
        end
    end
end

function Lua_PackBatchItem:onPressClick()
    if self.data == nil then
        return;
    end
    if self.selectedCount >= self.data.count and not Lua_WindowUtil.isWindowShowing("Lua_ItemDetailsWindow") then
        DataCenter.packUtil:showItemInfo(self.data);
    end
    if self.data.main_type == Lua_PackDefine.EQUIP and self.data.lockState then--装备上锁，不可选择分解
        OperateAlert.showLangValue(126783);
        return;
    end
    Lua_MsgUtil.callFunc("PACK_BATCH_PLUS", self.data);
end

function Lua_PackBatchItem:onLongPressClick()
    self:onPressClick();
end

function Lua_PackBatchItem:onBtnLessClick(go)
    if self.data == nil or self.selectedCount <= 0 then
        return;
    end
    self:startLessTimer();
end

function Lua_PackBatchItem:onBtnLessUpClick(go)
    if self.data == nil then
        return;
    end
    self:removeLessTimer();
end

function Lua_PackBatchItem:startLessTimer()
    if self.bLessTimer ~= true then
        self.bLessTimer = true;
        self.startLessTime = getTime();
        Lua_CommonUtil.addTimer("Lua_PackBatchItem", 0.01, self, self.onLessTimer);
    end
end

function Lua_PackBatchItem:removeLessTimer()
    if self.bLessTimer == true then
        self.bLessTimer = false;
        Lua_CommonUtil.removeTimer(self, self.onLessTimer);
        if not self.bLongPress then
            self:onLess();
        end
    end
end

function Lua_PackBatchItem:onLessTimer()
    if self.selectedCount <= 0 then
        self:removeLessTimer();
        return;
    end

    if getTime() - self.startLessTime > 0.3 then
        self.bLongPress = true;
        self:onLess();
    else
        self.bLongPress = false;
    end
end

function Lua_PackBatchItem:onLess()
    Lua_MsgUtil.callFunc("PACK_BATCH_LESS", self.data);
end




















Lua_PackBatchGetItem = class("Lua_PackBatchGetItem", Lua_ItemIconCell)
function Lua_PackBatchGetItem:buildShowItem()
    self:superCall("buildShowItem")
    self.itemIcon:setScale(0.9, 0.9, 0.9)
end