Lua_PackPanel = class("Lua_PackPanel", Lua_DisplayObject);
function Lua_PackPanel:ctor(p)
	self:superCall("ctor", p);
end

function Lua_PackPanel:buildPanel(go)
	self:superCall("buildPanel", go);
	self:initEvent();
end

function Lua_PackPanel:buildShowItem()
	self:superCall("buildShowItem");

	self.line = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_line_005", 0, -410, 0, 0);

	self.btnDeco = UICreate.createButton(self.content, nil, -172, -470, 220, 67);
	self.btnDeco:setStyle(3);
	self.btnDeco:setLangValue(309001)
	self.btnDeco:addClickFuntion(self, self.onBtnDecoClick);

	self.btnSell = UICreate.createButton(self.content, nil, 172, -470, 220, 67);
	self.btnSell:setStyle(1);
	self.btnSell:setLangValue(309002)
	self.btnSell:addClickFuntion(self, self.onBtnSellClick);

    self.itemList = Lua_DragList.new(self.content, 630, 820, Lua_AxisDefine.Vertical);
    self.itemList:setLocation(0, 22, 0);
    self.itemList:setListCell(Lua_PackItem, 140, 150, 4);
    self.itemList:setCellSpacing(20, 10);
end

function Lua_PackPanel:dispose()
	if self.itemList ~= nil then
		self.itemList:dispose();
		self.itemList = nil;
	end
	if self.itemArr ~= nil then
		self.itemArr:clear();
		self.itemArr = nil;
	end
	self.itemDic = nil;
	self.data = nil;
	self:superCall("dispose");
end

function Lua_PackPanel:initEvent()
	self:addLuaEvent("PACK_UPDATE", self, self.updateItem);
	self:addLuaEvent("ADD_PACK_ITEMS", self, self.addItem);
	self:addLuaEvent("REMOVE_PACK_ITEMS", self, self.removeItem);

	self:addLuaEvent("RUNES_ATTR", self, self.updateRunesInfo);--符石重铸
	self:addLuaEvent("DELETE_RUNES_ITEM", self, self.updateRunesInfo);--符石更新
end

function Lua_PackPanel:setData(data)
	self.data = data;
	self:updateShowInfo();

	if self.itemArr == nil then
		self.itemArr = Array();
	else
		self.itemArr:clear();
	end
	self.itemDic = nil;
	if data.package == Lua_PackDefine.TYPE_RUNSE then
		self:updateListInfo(DataCenter.runesData.runesPackInfo);
	else
		self.itemArr, self.itemDic = DataCenter.packData:getPackArrByPackType(data.package);
		self:updateListInfo(self.itemArr);
	end

	self.itemList:moveToIndex(1, true);
end

function Lua_PackPanel:updateShowInfo()
	local showDeco = false;
	local showSell = false;
	if self.data.btn_list ~= nil then
		for i = 1, #self.data.btn_list do
			if self.data.btn_list[i] == 1 then
				showDeco = true;
			elseif self.data.btn_list[i] == 2 then
				showSell = true;
			end
		end
	end

	if showDeco or showSell then
		self.itemList:setLocation(0, 22, 0);
		self.itemList:setContentRect(630, 820);
		self.line:setEnable(true);
	else
		self.itemList:setLocation(0, -36, 0);
		self.itemList:setContentRect(630, 940);
		self.line:setEnable(false);
	end
	
	self.btnDeco:setEnable(showDeco);
	self.btnSell:setEnable(showSell);
	if showDeco and not showSell then
		self.btnDeco:setLocation(0, -470, 0);
		self.btnDeco:setStyle(3);
	elseif not showDeco and showSell then
		self.btnSell:setLocation(0, -470, 0);
		self.btnSell:setStyle(3);
	else
		self.btnDeco:setLocation(-172, -470, 0);
		self.btnDeco:setStyle(3);
		self.btnSell:setLocation(172, -470, 0);
		self.btnSell:setStyle(1);
	end
end

function Lua_PackPanel:updateListInfo(arr)
	self:showNone(arr:getSize() <= 0);
	self.itemList:setArrayData(arr);
end

--符石数据更新
function Lua_PackPanel:updateRunesInfo()
	if not self.isActive or self.data == nil or self.data.package ~= Lua_PackDefine.TYPE_RUNSE then
		return;
	end

	self:updateListInfo(DataCenter.runesData.runesPackInfo);
end

function Lua_PackPanel:showNone(bShow)
	if bShow then
		if self.noneIcon == nil then
			self.noneIcon = UICreate.createUIAtlasImage(self.content, "common_altas_3", "xui_common_widget_012", 0, 50, 0, 0);
			self.noneText = UICreate.createLabel(self.content, nil, 0, 18.5, 100, 40, 40);
			self.noneText.pivotValue = 4;
			self.noneText.fontName = Lua_MyUISettings.huakangFontName;
			self.noneText.alignmentValue = 4;
			self.noneText.horizontalOverflowValue = 1;
			self.noneText.colorStr = Lua_ColorDefine.transColorStr("938680");
			self.noneText:setLangValue(309000);
		end
		self.noneIcon:setEnable(true);
		self.noneText:setEnable(true);
	else
		if self.noneIcon then
			self.noneIcon:setEnable(false);
			self.noneText:setEnable(false);
		end
	end
end

function Lua_PackPanel:updateItem(msg)
	if not self.isActive or self.data == nil or msg.type ~= self.data.package then
		return;
	end

	self.itemList:updateShowList();
end

function Lua_PackPanel:addItem(msg)
	if not self.isActive or self.data == nil or msg.type ~= self.data.package then
		return;
	end
	local list = msg.item_list;
	local unique_id;
	local index;
	-- local len = self.itemArr:getSize();
	-- for i = 1, #list do
	-- 	unique_id = tonumber(list[i].unique_id);
	-- 	index = self.itemDic[unique_id];
	-- 	if index == nil then -- 新增道具
	-- 		self.itemArr:add(unique_id);
	-- 		len = self.itemArr:getSize();
	-- 		self.itemDic[unique_id] = len;
	-- 	end
	-- end
	local len = #list;
	while len > 0 do
		unique_id = tonumber(list[len].unique_id);
		index = self.itemDic[unique_id];
		if index == nil then
			self.itemArr:insert(unique_id, 1);
		end
		len = len - 1;
	end
	len = self.itemArr:getSize();
	for i = 1, len do
		unique_id = self.itemArr[i]
		self.itemDic[unique_id] = i;
	end
	
	self:showNone(len <= 0);
	self.itemList:setArrayData(self.itemArr);
end

function Lua_PackPanel:removeItem(msg)
	if not self.isActive or self.data == nil or msg.type ~= self.data.package then
		return;
	end

	local list = msg.item_list;
	local unique_id;
	local index;
	local tvo;
	local len = self.itemArr:getSize();
	for i = 1, #list do
		unique_id = tonumber(list[i].unique_id);
		index = self.itemDic[unique_id];
		if index ~= nil then
			tvo = DataCenter.packData:getItemVoByUnId(unique_id);
			if tvo == nil then -- 删除道具
				self.itemArr:remove(index);
				self.itemDic[unique_id] = nil;
				len = self.itemArr:getSize();
				for j = index, len do
					unique_id = self.itemArr[j];
					self.itemDic[unique_id] = j;
				end
			end
		end
	end

	self:showNone(len <= 0);
	self.itemList:setArrayData(self.itemArr);
end

function Lua_PackPanel:onBtnDecoClick(go)
	Lua_WindowUtil.openWin("Lua_PackBatchWindow", Lua_WinParam.ACTION_OPEN, { packType = self.data.package, batchType = 1 });
end

function Lua_PackPanel:onBtnSellClick(go)
	Lua_WindowUtil.openWin("Lua_PackBatchWindow", Lua_WinParam.ACTION_OPEN, { packType = self.data.package, batchType = 2 });
end




















Lua_PackItem = class("Lua_PackItem", Lua_ListCell);
function Lua_PackItem:ctor(p)
	self:superCall("ctor", p);
end

function Lua_PackItem:buildShowItem()
	self:superCall("buildShowItem");
	self.bg = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_widget_009", 0, -5, 140, 140);
	self.itemIcon = Lua_ItemIcon.new(self.content);
	self.itemIcon:setLocation(0, -5, 0);
end

function Lua_PackItem:dispose()
	if self.itemIcon ~= nil then
		self.itemIcon:dispose();
		self.itemIcon = nil;
	end
	self._data = nil;
	self.data = nil;
	self:superCall("dispose");
end

function Lua_PackItem:setData(data)
	self._data = data;
	self:updateData();
end

function Lua_PackItem:updateData()
	if type(self._data) == "number" then
		if self._data == 0 then
			self.data = nil;
		else
			self.data = DataCenter.packData:getItemVoByUnId(self._data);
		end
	else
		self.data = self._data
	end

	self.itemIcon:setData(self.data);
	self.itemIcon:setDoubleIcon(nil);
	self:showUseLogo(self.data and self.data.use_logo == 1);
end

function Lua_PackItem:showUseLogo(value)
	if value then
		if self.useIcon == nil then
			self.useIcon = UICreate.createUIAtlasImage(self.content, "common_altas_1", "xui_common_label_11", 48.5, 56, 0, 0);
		end
		self.useIcon:setEnable(true);
	else
		if self.useIcon then
			self.useIcon:setEnable(false);
		end
	end
end

function Lua_PackItem:onPressClick()
	if self.data == nil or self.data.id == nil then
		return;
	end
	DataCenter.packUtil:showItemInfo(self.data, true, true);
end

function Lua_PackItem:onMouseDownHandle()
	self.itemIcon:setScale(0.95, 0.95, 0.95);
end

function Lua_PackItem:onMouseUpHandle()
	self.itemIcon:setScale(1, 1, 1);
end